@@ -1,8 +1,12 @@
 package URI::Fast;
-$URI::Fast::VERSION = '0.08';
+$URI::Fast::VERSION = '0.09';
 # ABSTRACT: A fast(er) URI parser
 
 use utf8;
+use strict;
+use warnings;
+no strict 'refs';
+
 use Carp;
 use Inline C => 'lib/uri_fast.c';
 
@@ -81,18 +85,17 @@ sub query {
   if (@_ == 2) {
     if (ref $val) {
       $self->set_query('', 1);
-
-      foreach (keys %$val) {
-        $self->param($_, $val->{$_});
-      }
+      $self->param($_, $val->{$_})
+        foreach keys %$val;
     }
     else {
       $self->set_query($val, 0);
     }
   }
 
-  return $self->get_query
-    if defined wantarray;
+  return unless defined(wantarray);         # void context
+  return $self->get_query unless wantarray; # scalar context
+  return %{ $self->query_hash };            # list context
 }
 
 sub query_keys {
@@ -136,7 +139,7 @@ URI::Fast - A fast(er) URI parser
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
